@@ -13,14 +13,18 @@ from typing import TYPE_CHECKING
 from requests import HTTPError, JSONDecodeError
 from urllib3.util.url import Url, parse_url
 
-from semantic_release.errors import UnexpectedResponse
+from semantic_release.errors import (
+    AssetUploadError,
+    IncompleteReleaseError,
+    UnexpectedResponse,
+)
 from semantic_release.helpers import logged_function
-from semantic_release.hvcs._base import HvcsBase
+from semantic_release.hvcs.remote_hvcs_base import RemoteHvcsBase
 from semantic_release.hvcs.token_auth import TokenAuth
 from semantic_release.hvcs.util import build_requests_session, suppress_not_found
 
 if TYPE_CHECKING:
-    from typing import Any
+    from typing import Any, Callable
 
 
 # Globals
@@ -45,7 +49,7 @@ if mimetypes.guess_type("test.md")[0] != "text/markdown":
     mimetypes.add_type("text/markdown", ".md")
 
 
-class Github(HvcsBase):
+class Github(RemoteHvcsBase):
     """
     GitHub HVCS interface for interacting with GitHub repositories
 
@@ -72,6 +76,9 @@ class Github(HvcsBase):
     DEFAULT_API_DOMAIN = f"{DEFAULT_API_SUBDOMAIN_PREFIX}.{DEFAULT_DOMAIN}"
     DEFAULT_API_PATH_CLOUD = "/"  # no path prefix!
     DEFAULT_API_PATH_ONPREM = "/api/v3"
+    DEFAULT_API_URL_CLOUD = f"https://{DEFAULT_API_SUBDOMAIN_PREFIX}.{DEFAULT_DOMAIN}{DEFAULT_API_PATH_CLOUD}".rstrip(
+        "/"
+    )
     DEFAULT_ENV_TOKEN_NAME = "GH_TOKEN"  # noqa: S105
 
     def __init__(
@@ -90,27 +97,19 @@ class Github(HvcsBase):
         self.session = build_requests_session(auth=auth)
 
         # ref: https://docs.github.com/en/actions/reference/environment-variables#default-environment-variables
-        domain_url = parse_url(
+        domain_url_str = (
             hvcs_domain
             or os.getenv("GITHUB_SERVER_URL", "")
             or f"https://{self.DEFAULT_DOMAIN}"
         )
 
-        if domain_url.scheme == "http" and not allow_insecure:
-            raise ValueError("Insecure connections are currently disabled.")
-
-        if not domain_url.scheme:
-            new_scheme = "http" if allow_insecure else "https"
-            domain_url = Url(**{**domain_url._asdict(), "scheme": new_scheme})
-
-        if domain_url.scheme not in ["http", "https"]:
-            raise ValueError(
-                f"Invalid scheme {domain_url.scheme} for domain {domain_url.host}. "
-                "Only http and https are supported."
-            )
+        domain_url = self._normalize_url(
+            domain_url_str,
+            allow_insecure=allow_insecure,
+        )
 
         # Strip any auth, query or fragment from the domain
-        self.hvcs_domain = parse_url(
+        self._hvcs_domain = parse_url(
             Url(
                 scheme=domain_url.scheme,
                 host=domain_url.host,
@@ -120,10 +119,66 @@ class Github(HvcsBase):
         )
 
         # ref: https://docs.github.com/en/actions/reference/environment-variables#default-environment-variables
-        api_domain_parts = parse_url(
+        api_url_str = (
             hvcs_api_domain
             or os.getenv("GITHUB_API_URL", "")
-            or Url(
+            or self._derive_api_url_from_base_domain()
+        )
+
+        api_domain_parts = self._normalize_url(
+            api_url_str,
+            allow_insecure=allow_insecure,
+        )
+
+        # As GitHub Enterprise Cloud and GitHub Enterprise Server (on-prem) have different api locations
+        # lets check what we have been given and set the api url accordingly
+        #   NOTE: Github Server (on premise) uses a path prefix '/api/v3' for the api
+        #         while GitHub Enterprise Cloud uses a separate subdomain as the base
+        is_github_cloud = bool(self.hvcs_domain.url == f"https://{self.DEFAULT_DOMAIN}")
+
+        if (
+            is_github_cloud
+            and hvcs_api_domain
+            and api_domain_parts.url not in Github.DEFAULT_API_URL_CLOUD
+        ):
+            # Api was provied but is not a subset of the expected one, raise an error
+            # we check for a subset because the user may not have provided the full api path
+            # but the correct domain.  If they didn't, then we are erroring out here.
+            raise ValueError(
+                f"Invalid api domain {api_domain_parts.url} for GitHub Enterprise Cloud. "
+                f"Expected {Github.DEFAULT_API_URL_CLOUD}."
+            )
+
+        # Set the api url to the default cloud one if we are on cloud, otherwise
+        # use the verified api domain for a on-prem server
+        self._api_url = parse_url(
+            Github.DEFAULT_API_URL_CLOUD
+            if is_github_cloud
+            else Url(
+                # Strip any auth, query or fragment from the domain
+                scheme=api_domain_parts.scheme,
+                host=api_domain_parts.host,
+                port=api_domain_parts.port,
+                path=str(
+                    PurePosixPath(
+                        # pass any custom server prefix path but ensure we don't
+                        # double up the api path in the case the user provided it
+                        str.replace(
+                            api_domain_parts.path or "",
+                            self.DEFAULT_API_PATH_ONPREM,
+                            "",
+                        ).lstrip("/")
+                        or "/",
+                        # apply the on-prem api path
+                        self.DEFAULT_API_PATH_ONPREM.lstrip("/"),
+                    )
+                ),
+            ).url.rstrip("/")
+        )
+
+    def _derive_api_url_from_base_domain(self) -> Url:
+        return parse_url(
+            Url(
                 # infer from Domain url and prepend the default api subdomain
                 **{
                     **self.hvcs_domain._asdict(),
@@ -138,81 +193,6 @@ class Github(HvcsBase):
             ).url.rstrip("/")
         )
 
-        if api_domain_parts.scheme == "http" and not allow_insecure:
-            raise ValueError("Insecure connections are currently disabled.")
-
-        if not api_domain_parts.scheme:
-            new_scheme = "http" if allow_insecure else "https"
-            api_domain_parts = Url(
-                **{**api_domain_parts._asdict(), "scheme": new_scheme}
-            )
-
-        if api_domain_parts.scheme not in ["http", "https"]:
-            raise ValueError(
-                f"Invalid scheme {api_domain_parts.scheme} for api domain {api_domain_parts.host}. "
-                "Only http and https are supported."
-            )
-
-        # As GitHub Enterprise Cloud and GitHub Enterprise Server (on-prem) have different api locations
-        # lets check what we have been given and set the api url accordingly
-        #   NOTE: Github Server (on premise) uses a path prefix '/api/v3' for the api
-        #         while GitHub Enterprise Cloud uses a separate subdomain as the base
-        is_github_cloud = bool(self.hvcs_domain.url == f"https://{self.DEFAULT_DOMAIN}")
-
-        # Calculate out the api url that we expect for GitHub Cloud
-        default_cloud_api_url = parse_url(
-            Url(
-                # set api domain and append the default api path
-                **{
-                    **self.hvcs_domain._asdict(),
-                    "host": f"{self.DEFAULT_API_DOMAIN}",
-                    "path": self.DEFAULT_API_PATH_CLOUD,
-                }
-            ).url.rstrip("/")
-        )
-
-        if (
-            is_github_cloud
-            and hvcs_api_domain
-            and api_domain_parts.url not in default_cloud_api_url.url
-        ):
-            # Api was provied but is not a subset of the expected one, raise an error
-            # we check for a subset because the user may not have provided the full api path
-            # but the correct domain.  If they didn't, then we are erroring out here.
-            raise ValueError(
-                f"Invalid api domain {api_domain_parts.url} for GitHub Enterprise Cloud. "
-                f"Expected {default_cloud_api_url.url}."
-            )
-
-        # Set the api url to the default cloud one if we are on cloud, otherwise
-        # use the verified api domain for a on-prem server
-        self.api_url = (
-            default_cloud_api_url
-            if is_github_cloud
-            else parse_url(
-                # Strip any auth, query or fragment from the domain
-                Url(
-                    scheme=api_domain_parts.scheme,
-                    host=api_domain_parts.host,
-                    port=api_domain_parts.port,
-                    path=str(
-                        PurePosixPath(
-                            # pass any custom server prefix path but ensure we don't
-                            # double up the api path in the case the user provided it
-                            str.replace(
-                                api_domain_parts.path or "",
-                                self.DEFAULT_API_PATH_ONPREM,
-                                "",
-                            ).lstrip("/")
-                            or "/",
-                            # apply the on-prem api path
-                            self.DEFAULT_API_PATH_ONPREM.lstrip("/"),
-                        )
-                    ),
-                ).url.rstrip("/")
-            )
-        )
-
     @lru_cache(maxsize=1)
     def _get_repository_owner_and_name(self) -> tuple[str, str]:
         # Github actions context
@@ -223,27 +203,27 @@ class Github(HvcsBase):
 
         return super()._get_repository_owner_and_name()
 
-    def compare_url(self, from_rev: str, to_rev: str) -> str:
-        """
-        Get the GitHub comparison link between two version tags.
-        :param from_rev: The older version to compare.
-        :param to_rev: The newer version to compare.
-        :return: Link to view a comparison between the two versions.
-        """
-        return self.create_server_url(
-            path=f"/{self.owner}/{self.repo_name}/compare/{from_rev}...{to_rev}"
-        )
-
     @logged_function(log)
     def create_release(
-        self, tag: str, release_notes: str, prerelease: bool = False
+        self,
+        tag: str,
+        release_notes: str,
+        prerelease: bool = False,
+        assets: list[str] | None = None,
     ) -> int:
         """
         Create a new release
-        https://docs.github.com/rest/reference/repos#create-a-release
+
+        REF: https://docs.github.com/rest/reference/repos#create-a-release
+
         :param tag: Tag to create release for
+
         :param release_notes: The release notes for this version
+
         :param prerelease: Whether or not this release should be created as a prerelease
+
+        :param assets: a list of artifacts to upload to the release
+
         :return: the ID of the release
         """
         log.info("Creating release for tag %s", tag)
@@ -267,12 +247,33 @@ class Github(HvcsBase):
         try:
             release_id: int = response.json()["id"]
             log.info("Successfully created release with ID: %s", release_id)
-            return release_id
         except JSONDecodeError as err:
             raise UnexpectedResponse("Unreadable json response") from err
         except KeyError as err:
             raise UnexpectedResponse("JSON response is missing an id") from err
 
+        errors = []
+        for asset in assets or []:
+            log.info("Uploading asset %s", asset)
+            try:
+                self.upload_release_asset(release_id, asset)
+            except HTTPError as err:
+                errors.append(
+                    AssetUploadError(f"Failed asset upload for {asset}").with_traceback(
+                        err.__traceback__
+                    )
+                )
+
+        if len(errors) < 1:
+            return release_id
+
+        for error in errors:
+            log.exception(error)
+
+        raise IncompleteReleaseError(
+            f"Failed to upload asset{'s' if len(errors) > 1 else ''} to release!"
+        )
+
     @logged_function(log)
     @suppress_not_found
     def get_release_id_by_tag(self, tag: str) -> int | None:
@@ -377,7 +378,7 @@ class Github(HvcsBase):
             ) from err
 
     @logged_function(log)
-    def upload_asset(
+    def upload_release_asset(
         self, release_id: int, file: str, label: str | None = None
     ) -> bool:
         """
@@ -442,7 +443,7 @@ class Github(HvcsBase):
             f for f in glob.glob(dist_glob, recursive=True) if os.path.isfile(f)
         ):
             try:
-                self.upload_asset(release_id, file_path)
+                self.upload_release_asset(release_id, file_path)
                 n_succeeded += 1
             except HTTPError:  # noqa: PERF203
                 log.exception("error uploading asset %s", file_path)
@@ -461,70 +462,33 @@ class Github(HvcsBase):
             path=f"/{self.owner}/{self.repo_name}.git",
         )
 
+    def compare_url(self, from_rev: str, to_rev: str) -> str:
+        """
+        Get the GitHub comparison link between two version tags.
+        :param from_rev: The older version to compare.
+        :param to_rev: The newer version to compare.
+        :return: Link to view a comparison between the two versions.
+        """
+        return self.create_repo_url(repo_path=f"/compare/{from_rev}...{to_rev}")
+
     def commit_hash_url(self, commit_hash: str) -> str:
-        return self.create_server_url(
-            path=f"/{self.owner}/{self.repo_name}/commit/{commit_hash}"
-        )
+        return self.create_repo_url(repo_path=f"/commit/{commit_hash}")
+
+    def issue_url(self, issue_num: str | int) -> str:
+        return self.create_repo_url(repo_path=f"/issues/{issue_num}")
 
     def pull_request_url(self, pr_number: str | int) -> str:
-        return self.create_server_url(
-            path=f"/{self.owner}/{self.repo_name}/issues/{pr_number}"
+        return self.create_repo_url(repo_path=f"/pull/{pr_number}")
+
+    def get_changelog_context_filters(self) -> tuple[Callable[..., Any], ...]:
+        return (
+            self.create_server_url,
+            self.create_repo_url,
+            self.commit_hash_url,
+            self.compare_url,
+            self.issue_url,
+            self.pull_request_url,
         )
 
-    def _derive_url(
-        self,
-        base_url: Url,
-        path: str,
-        auth: str | None = None,
-        query: str | None = None,
-        fragment: str | None = None,
-    ) -> str:
-        overrides = dict(
-            filter(
-                lambda x: x[1] is not None,
-                {
-                    "auth": auth,
-                    "path": str(PurePosixPath("/", path.lstrip("/"))),
-                    "query": query,
-                    "fragment": fragment,
-                }.items(),
-            )
-        )
-        return Url(
-            **{
-                **base_url._asdict(),
-                **overrides,
-            }
-        ).url.rstrip("/")
 
-    def create_server_url(
-        self,
-        path: str,
-        auth: str | None = None,
-        query: str | None = None,
-        fragment: str | None = None,
-    ) -> str:
-        # Ensure any path prefix is transfered but not doubled up on the derived url
-        return self._derive_url(
-            self.hvcs_domain,
-            path=f"{self.hvcs_domain.path or ''}/{path.lstrip(self.hvcs_domain.path)}",
-            auth=auth,
-            query=query,
-            fragment=fragment,
-        )
-
-    def create_api_url(
-        self,
-        endpoint: str,
-        auth: str | None = None,
-        query: str | None = None,
-        fragment: str | None = None,
-    ) -> str:
-        # Ensure any api path prefix is transfered but not doubled up on the derived api url
-        return self._derive_url(
-            self.api_url,
-            path=f"{self.api_url.path or ''}/{endpoint.lstrip(self.api_url.path)}",
-            auth=auth,
-            query=query,
-            fragment=fragment,
-        )
+RemoteHvcsBase.register(Github)
